
//import org.apache.xerces.parser.DOMParser;
import org.w3c.dom.Document; 
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.NamedNodeMap; 
import org.w3c.dom.Element; 
import org.w3c.dom.Text; 
import org.w3c.dom.Comment; 
import org.w3c.dom.EntityReference; 
import org.w3c.dom.DocumentType; 
import java.io.IOException;
import java.io.PrintWriter;
import org.xml.sax.SAXException;


public class DOMSchreiber {

    PrintWriter pw = null;

    public DOMSchreiber() {
	pw = new Screen();
    }

    public DOMSchreiber(PrintWriter p) {
	pw = p;
    }

    public void printNode(Node node) {
	switch ( node.getNodeType() ) {
	case Node.ELEMENT_NODE: 
            printElement( (Element)node );
	    break;
	case Node.ATTRIBUTE_NODE: 
            printAttributes( (NamedNodeMap)node );
	    break;
	case Node.TEXT_NODE: 
            printText( (Text)node );
	    break;
	case Node.CDATA_SECTION_NODE: 
            printCData( (Text)node );
	    break;
	case Node.PROCESSING_INSTRUCTION_NODE: 
            printPI( node );
	    break;
	case Node.COMMENT_NODE: 
            printComment( (Comment)node );
	    break;
	case Node.DOCUMENT_TYPE_NODE: 
            printDoctype( (DocumentType)node );
	    break;
	case Node.DOCUMENT_NODE: 
            printDocument( (Document)node );
	    break;
	case Node.ENTITY_REFERENCE_NODE: 
            printEntityReference( (EntityReference)node );
	    break;
	default: pw.println("<!-- " + node + " -->");
	}
    }


    public void printElement(Element el) {
        String name = el.getNodeName();
        pw.print("<" +  name );
        printAttributes( el.getAttributes() );
        NodeList nl = el.getChildNodes();
        if ( ( nl == null ) || ( nl.getLength() == 0 ) ) {
	    pw.print(" />");
            return;
	}
        pw.print(">"); 
	for (int i = 0; i < nl.getLength(); i++ ){
	    printNode(nl.item(i));
	}
        pw.print("</" + name + ">");

    }


    public void printAttributes(NamedNodeMap at) {
        if ( at == null ) return;
        for ( int i = 0; i < at.getLength(); i++ ){
            Node an = at.item(i);
            pw.print(" " + an.getNodeName() + "=\"");
            pw.print(an.getNodeValue() + "\"");
	}
    }


    public void printText(Text tx) {
	pw.print( tx.getData() );
    }


    public void printEntityReference(EntityReference er) {
	pw.print("&" + er.getNodeName() + ";" );
    }


    public void printComment(Comment c) {
	pw.print("<!--");
	pw.print( c.getData() );
	pw.print("-->");
    }


    public void printDoctype(DocumentType d) {
	pw.print("<!DOCTYPE ");
	pw.print( d.getName() );
        String pid = d.getPublicId();
	if ( pid != null ) {
	    pw.print(" PUBLIC \"" + pid + "\"\n          ");
	} else {
	    pw.print(" SYSTEM " );
	}
        pw.print("\"" + d.getSystemId() + "\" ");
	pw.println(">");
    }


    public void printCData(Text tx) {
	pw.print("<![CDATA[ ");
	pw.print( tx.getData() );
	pw.print("]]>");
    }


    public void printDocument(Document doc) {
	pw.println("<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>");
        printNode( doc.getDoctype() );
        printNode( doc.getDocumentElement() );
	pw.println();
    }


    public void printPI(Node pi) {
	pw.print("<?" + pi.getNodeName() );
	pw.print(" " + pi.getNodeValue() + "?>" );
    }


    public void flush() {
	pw.flush();
    }


    public void close() {
	pw.close();
    }

}
